/**************************************************
 * sqi
 *
 **************************************************
 * This code was created by Peter Harvey @ CodeByDesign.
 * Released under GPL 18.FEB.99
 *
 * Contributions from...
 * -----------------------------------------------
 * Peter Harvey		- pharvey@codebydesign.com
 **************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <sqi.h>
#include <ini.h>

char *szSyntax =
"\n" \
"**********************************************\n" \
"* unixODBC - sqi                             *\n" \
"**********************************************\n" \
"* Syntax                                     *\n" \
"*                                            *\n" \
"*      sqi DatabaseFile                      *\n" \
"*                                            *\n" \
"* Options                                    *\n" \
"*                                            *\n" \
"* -b         Batch. No prompting etc         *\n" \
"* -dx        Delimit cols with x             *\n" \
"* -w         Wrap results in an HTML table   *\n" \
"* -v         Verbose.                        *\n" \
"*                                            *\n" \
"* Notes                                      *\n" \
"*                                            *\n" \
"*      sqi supports redirection and piping   *\n" \
"*      for batch processing.                 *\n" \
"*                                            *\n" \
"* Examples                                   *\n" \
"*                                            *\n" \
"*      cat My.sql | sqi contacts.db -w       *\n" \
"*                                            *\n" \
"*      Each line in My.sql must contain      *\n" \
"*      exactly 1 SQL command except for the  *\n" \
"*      last line which must be blank.        *\n" \
"*                                            *\n" \
"* Please visit;                              *\n" \
"*                                            *\n" \
"*      http://www.unixODBC.org               *\n" \
"*      pharvey@codebydesign.com              *\n" \
"**********************************************\n\n";

#define MAX_DATA_WIDTH 300

#ifndef max
#define max( a, b ) ((a > b ) ? a : b)
#endif

int OpenDatabase( HSQI *phSQI, char *szDatabase );
int ExecuteSQL( HSQI hSQI, char *szSQL, char cDelimiter, int bHTMLTable );
int ExecuteHelp( HSQI hSQI, char *szSQL, char cDelimiter, int bHTMLTable );
int	CloseDatabase( HSQI hSQI );

void WriteHeaderHTMLTable( HSQISTMT hStmt );
void WriteHeaderNormal( HSQISTMT hStmt, char *szSepLine );
void WriteBodyHTMLTable( HSQISTMT hStmt );
void WriteBodyNormal( HSQISTMT hStmt );
void WriteBodyDelimited( HSQISTMT hStmt, char cDelimiter );
void WriteFooterHTMLTable( HSQISTMT hStmt );
void WriteFooterNormal( HSQISTMT hStmt, char *szSepLine );

void DumpLog( HLOG hLog );

/**************************************************
 * sqi
 *
 **************************************************
 * This code was created by Peter Harvey @ CodeByDesign.
 * Released under GPL 18.FEB.99
 *
 * Contributions from...
 * -----------------------------------------------
 * Peter Harvey		- pharvey@codebydesign.com
 **************************************************/

int 	bVerbose					= 0;

int main( int argc, char *argv[] )
{
	int 	nArg, count;
	int 	bHTMLTable						= 0;
	int		bBatch							= 0;
	int 	cDelimiter						= 0;
	char	szDatabase[MAX_DATA_WIDTH+1]	= "";
	char	szSQL[9001]						= "";
	char    *pEscapeChar;
	HSQI	hSQI							= 0;

	if ( argc < 2 )
	{
		fprintf( stderr, szSyntax );
		exit( 1 );
	}

	/****************************
	 * PARSE ARGS
	 ***************************/
	for ( nArg = 1, count = 1 ; nArg < argc; nArg++ )
	{
		if ( argv[nArg][0] == '-' )
		{
			/* Options */
			switch ( argv[nArg][1] )
			{
			case 'd':
				cDelimiter = argv[nArg][2];
				break;
			case 'w':
				bHTMLTable = 1;
				break;
			case 'b':
				bBatch = 1;
				break;
			case 'v':
				bVerbose = 1;
				break;
			default:
				fprintf( stderr, szSyntax );
				exit( 1 );
			}
			continue;
		}
		else if ( count == 1 )
			strncpy( szDatabase, argv[nArg], MAX_DATA_WIDTH );
		count++;
	}

	/****************************
	 * CONNECT
	 ***************************/
	if ( !OpenDatabase( &hSQI, szDatabase ) )
		exit( 1 );

	if ( bVerbose )
		DumpLog( hSQI->hLog );

	/****************************
	 * EXECUTE
	 ***************************/
	if ( !bBatch )
	{
		printf( "+---------------------------------------+\n" );
		printf( "| Connected!                            |\n" );
		printf( "|                                       |\n" );
		printf( "| sql_command                           |\n" );
		printf( "| help [tablename]                      |\n" );
		printf( "| quit                                  |\n" );
		printf( "|                                       |\n" );
		printf( "+---------------------------------------+\n" );
	}
	do
	{
		if ( !bBatch )
			printf( "SQI> " );

		fgets( szSQL, sizeof(szSQL), stdin );

		/* strip away escape chars */
		while ( (pEscapeChar=(char*)strchr(szSQL, '\n')) != NULL || (pEscapeChar=(char*)strchr(szSQL, '\r')) != NULL )
			*pEscapeChar = ' ';

		if ( szSQL[1] != '\0' )
		{
			if ( strncmp( szSQL, "quit", 4 ) == 0 )
				szSQL[1] = '\0';
			else if ( strncmp( szSQL, "help", 4 ) == 0 )
				ExecuteHelp( hSQI, szSQL, cDelimiter, bHTMLTable );
			else
				ExecuteSQL( hSQI, szSQL, cDelimiter, bHTMLTable );
		
			if ( bVerbose )
				DumpLog( hSQI->hLog );
		}

	} while ( szSQL[1] != '\0' );

	/****************************
	 * DISCONNECT
	 ***************************/
	CloseDatabase( hSQI );

	exit( 0 );
}

/****************************
 * OpenDatabase - do everything we have to do to get a viable connection to szDatabase
 ***************************/
int OpenDatabase( HSQI *phSQI, char *szDatabase )
{

	if ( !sqiOpen( phSQI, szDatabase ) )
	{
		fprintf( stderr, "[SQI]ERROR: Could not sqiOpen %s\n", szDatabase );
		return 0;
	}

	return 1;
}

/****************************
 * ExecuteSQL - create a statement, execute the SQL, and get rid of the statement
 *            - show results as per request; bHTMLTable has precedence over other options
 ***************************/
int ExecuteSQL( HSQI hSQI, char *szSQL, char cDelimiter, int bHTMLTable )
{
	HSQISTMT 		hStmt;
	char			szSepLine[32001]		= "";	

	if ( !hSQI )
		return 0;

	/****************************
	 * EXECUTE SQL
	 ***************************/
	if ( !sqiOpenStmt( hSQI, &hStmt ) )
	{
		fprintf( stderr, "[SQI]ERROR: Could not sqiOpenStmt\n" );
		return 0;
	}

	if ( !sqiPrepare( hStmt, szSQL ) )
	{
		fprintf( stderr, "[SQI]ERROR: Could not sqiPrepare\n" );
		sqiCloseStmt( hStmt );
		return 0;
	}

	if ( !sqiExecute( hStmt ) )
	{
		fprintf( stderr, "[ISQL]ERROR: Could not sqiExecute\n" );
		sqiCloseStmt( hStmt );
		return 0;
	}

	/****************************
	 * WRITE HEADER
	 ***************************/
	if ( bHTMLTable )
		WriteHeaderHTMLTable( hStmt );
	else if ( cDelimiter == 0 )
		WriteHeaderNormal( hStmt, szSepLine );

	/****************************
	 * WRITE BODY
	 ***************************/
	if ( bHTMLTable )
		WriteBodyHTMLTable( hStmt );
	else if ( cDelimiter == 0 )
		WriteBodyNormal( hStmt );
	else
		WriteBodyDelimited( hStmt, cDelimiter );

	/****************************
	 * WRITE FOOTER
	 ***************************/
	if ( bHTMLTable )
		WriteFooterHTMLTable( hStmt );
	else if ( cDelimiter == 0 )
		WriteFooterNormal( hStmt, szSepLine );

	/****************************
	 * CLEANUP
	 ***************************/
	sqiCloseStmt( hStmt );

	return 1;
}

/****************************
 * ExecuteHelp - create a statement, execute the SQL, and get rid of the statement
 *             - show results as per request; bHTMLTable has precedence over other options
 ***************************/
int ExecuteHelp( HSQI hSQI, char *szSQL, char cDelimiter, int bHTMLTable )
{
	char 			szTable[250]						= "";
	HSQISTMT 		hStmt;
	char			szSepLine[32001]					= "";	

	/****************************
	 * EXECUTE SQL
	 ***************************/
	if ( !sqiOpenStmt( hSQI, &hStmt ) )
	{
		fprintf( stderr, "[SQI]ERROR: Could not sqiOpen\n" );
		return 0;
	}

	if ( iniElement( szSQL, ' ', '\0', 1, szTable, sizeof(szTable) ) == INI_SUCCESS )
	{
		/* COLUMNS */
		if ( !sqiColumns( hStmt, szTable ) )
		{
			fprintf( stderr, "[SQI]ERROR: Could not sqiColumns\n" );
			sqiCloseStmt( hStmt );
			return 0;
		}
	}
	else
	{
		/* TABLES */
		if ( !sqiTables( hStmt ) )
		{
			fprintf( stderr, "[SQI]ERROR: Could not sqiTables\n" );
			sqiCloseStmt( hStmt );
			return 0;
		}
	}

	/****************************
	 * WRITE HEADER
	 ***************************/
	if ( bHTMLTable )
		WriteHeaderHTMLTable( hStmt );
	else if ( cDelimiter == 0 )
		WriteHeaderNormal( hStmt, szSepLine );

	/****************************
	 * WRITE BODY
	 ***************************/
	if ( bHTMLTable )
		WriteBodyHTMLTable( hStmt );
	else if ( cDelimiter == 0 )
		WriteBodyNormal( hStmt );
	else
		WriteBodyDelimited( hStmt, cDelimiter );

	/****************************
	 * WRITE FOOTER
	 ***************************/
	if ( bHTMLTable )
		WriteFooterHTMLTable( hStmt );
	else if ( cDelimiter == 0 )
		WriteFooterNormal( hStmt, szSepLine );

	/****************************
	 * CLEANUP
	 ***************************/
	sqiCloseStmt( hStmt );

	return 1;
}


/****************************
 * CloseDatabase - cleanup in prep for exiting the program
 ***************************/
int	CloseDatabase( HSQI hSQI )
{
	sqiClose( hSQI );

	return 1;
}


/****************************
 * WRITE HTML
 ***************************/
void WriteHeaderHTMLTable( HSQISTMT hStmt )
{
	HSQICOLUMN		pColumnHeader;

	if ( !hStmt )
		return;
	if ( !hStmt->hTableResult )
		return;

	printf( "<table BORDER>\n" );
	printf( "<tr BGCOLOR=#000099>\n" );

	/* PROCESS ALL COLUMNS */
	lstFirst( hStmt->hTableResult->hColHdrs );
	while ( !lstEOL( hStmt->hTableResult->hColHdrs ) )
	{
		/* GET COLUMN NAME */
		pColumnHeader 	= (HSQICOLUMN)lstGet( hStmt->hTableResult->hColHdrs );
		
		/* FORMAT          */
		printf( "<td>\n" );
		printf( "<font face=Arial,Helvetica><font color=#FFFFFF>\n" );
		printf( "%s\n", pColumnHeader->pszSQL_DESC_LABEL );
		printf( "</font></font>\n" );
		printf( "</td>\n" );
	
		lstNext( hStmt->hTableResult->hColHdrs );
	}
	printf( "</tr>\n" );
}

void WriteBodyHTMLTable( HSQISTMT hStmt )
{
	HSQICOLUMN	pColumn;
	char		**aCols;
	char    	*pSourceData    = NULL;

	if ( !hStmt )
		return;
	if ( !hStmt->hTableResult )
		return;

    /* FETCH ROWS */
	lstFirst( hStmt->hTableResult->hRows );
	while( !lstEOL( hStmt->hTableResult->hRows ) )
    {
		printf( "<tr>\n" );
		aCols	= (char**)lstGet( hStmt->hTableResult->hRows );
		
		/* FETCH COLS */
		lstFirst( hStmt->hTableResult->hColHdrs );
        while( !lstEOL( hStmt->hTableResult->hColHdrs ) )
        {
			printf( "<td>\n" );
			printf( "<font face=Arial,Helvetica>\n" );

			pColumn = (HSQICOLUMN)lstGet( hStmt->hTableResult->hColHdrs );
			pSourceData = aCols[pColumn->nCol];
            
			if ( pSourceData )
                printf( "%s\n", pSourceData );
			else
				printf( "%s\n", "" );
	
			printf( "</font>\n" );
			printf( "</td>\n" );

			lstNext( hStmt->hTableResult->hColHdrs );
		}
		printf( "</tr>\n" );

		lstNext( hStmt->hTableResult->hRows );
    }
}

void WriteFooterHTMLTable( HSQISTMT hStmt )
{
	if ( !hStmt )
		return;
	if ( !hStmt->hTableResult )
		return;

	printf( "</table>\n" );
}

/****************************
 * WRITE DELIMITED
 ***************************/
void WriteBodyDelimited( HSQISTMT hStmt, char cDelimiter )
{
	HSQICOLUMN	pColumn;
	char		**aCols;
	char    	*pSourceData    = NULL;

	if ( !hStmt )
		return;
	if ( !hStmt->hTableResult )
		return;

	/* ROWS */
	lstFirst( hStmt->hTableResult->hRows );
    while( !lstEOL( hStmt->hTableResult->hRows ) )
    {
		aCols	= (char**)lstGet( hStmt->hTableResult->hRows );
		
		/* COLS */
        while ( !lstEOL( hStmt->hTableResult->hColHdrs ) )
        {

			pColumn = (HSQICOLUMN)lstGet( hStmt->hTableResult->hColHdrs );
			pSourceData = aCols[pColumn->nCol];
            
			if ( pSourceData )
                printf( "%s%c", pSourceData, cDelimiter );
            else
                printf( "%s%c", "", cDelimiter );

			lstNext( hStmt->hTableResult->hColHdrs );
        }
        printf( "\n" );
    
		lstNext( hStmt->hTableResult->hRows );
	}
}

/****************************
 * WRITE NORMAL
 ***************************/
void WriteHeaderNormal( HSQISTMT hStmt, char *szSepLine )
{
	char			szColumn[MAX_DATA_WIDTH+20]		= "";	
	char			szColumnName[MAX_DATA_WIDTH+1]	= "";	
	char			szHdrLine[32001]				= "";	
	unsigned int	nMaxLength						= 10;
	HSQICOLUMN		pColumnHeader;
	
	if ( !hStmt )
		return;
	if ( !hStmt->hTableResult )
		return;

	/* COLS */
	lstFirst( hStmt->hTableResult->hColHdrs );
	while ( !lstEOL( hStmt->hTableResult->hColHdrs ) )
	{
		pColumnHeader 	= (HSQICOLUMN)lstGet( hStmt->hTableResult->hColHdrs );
		
		strncpy( szColumnName, pColumnHeader->pszSQL_DESC_LABEL, MAX_DATA_WIDTH );

		nMaxLength 	= pColumnHeader->nSQL_DESC_DISPLAY_SIZE;
		if ( nMaxLength > MAX_DATA_WIDTH ) nMaxLength = MAX_DATA_WIDTH;

		/* SEP */
        memset( szColumn, '\0', sizeof(szColumn) );
		memset( szColumn, '-', max( nMaxLength, strlen(szColumnName) ) + 1 );
		strcat( szSepLine, "+" );
		strcat( szSepLine, szColumn );

		/* HDR */
		sprintf( szColumn, "| %-*s", max( nMaxLength, strlen(szColumnName) ), szColumnName );
        strcat( szHdrLine, szColumn );
	
		lstNext( hStmt->hTableResult->hColHdrs );	
	}
	strcat( szSepLine, "+\n" );
	strcat( szHdrLine, "|\n" );
	
	printf( szSepLine );
	printf( szHdrLine );
	printf( szSepLine );

}


void WriteBodyNormal( HSQISTMT hStmt )
{
	char			szColumn[MAX_DATA_WIDTH+20]		= "";
	char			szColumnName[MAX_DATA_WIDTH+1]	= "";	
	HSQICOLUMN		pColumnHeader;
	char			**aCols;
	char			*pSourceData    				= NULL;
	unsigned int	nMaxLength						= 10;
	
	if ( !hStmt )
		return;
	if ( !hStmt->hTableResult )
		return;

	/* ROWS */
	lstFirst( hStmt->hTableResult->hRows );
    while ( !lstEOL( hStmt->hTableResult->hRows ) )
    {
		aCols	= (char**)lstGet( hStmt->hTableResult->hRows );
		
		/* COLS */
		lstFirst( hStmt->hTableResult->hColHdrs );
        while ( !lstEOL( hStmt->hTableResult->hColHdrs ) )
        {
			pColumnHeader 	= (HSQICOLUMN)lstGet( hStmt->hTableResult->hColHdrs );

			strncpy( szColumnName, pColumnHeader->pszSQL_DESC_LABEL, MAX_DATA_WIDTH );

			nMaxLength 	= pColumnHeader->nSQL_DESC_DISPLAY_SIZE;
			if ( nMaxLength > MAX_DATA_WIDTH ) nMaxLength = MAX_DATA_WIDTH;

			pSourceData = aCols[pColumnHeader->nCol];

            if ( pSourceData )
                sprintf( szColumn, "| %-*s", max( nMaxLength, strlen(szColumnName) ), pSourceData );
            else
                sprintf( szColumn, "| %-*s", max( nMaxLength, strlen(szColumnName) ), "" );
			
			printf( szColumn );
			lstNext( hStmt->hTableResult->hColHdrs );
        }
        
		printf( "|\n" );
		lstNext( hStmt->hTableResult->hRows );
    } 
}

void WriteFooterNormal( HSQISTMT hStmt, char *szSepLine )
{
	if ( !hStmt )
		return;
	if ( !hStmt->hTableResult )
		return;

	printf( szSepLine );
	printf( "%ld rows affected\n", hStmt->hTableResult->hRows->nItems );
}


void DumpLog( HLOG hLog )
{
	char 	szMsgHdr[1024]	= "";
	char 	szMsg[1024]		= "";
	int		nCode			= -1;

	while ( logPopMsg( hLog, szMsgHdr, &nCode, szMsg ) == LOG_SUCCESS )
	{
		printf( "%s%s\n", szMsgHdr, szMsg );
		szMsgHdr[0] = '\0';
		szMsg[0]	= '\0';
		nCode		= -1;
	}
}



