#include "sqi.h"

#ifdef HAVE_PWD_H
#include <pwd.h>
#endif

/********************************************
 * sqiOpen
 *
 ********************************************/
int sqiOpen( HSQI *phSQI, char *pszCatalogFile )
{
	HSQI 	hSQI;
	HLOG	hLog;
	uid_t			nUserID						= getuid();
	struct passwd	*pPasswd					= NULL;
	char			*pHomeDir					= NULL;
    char szCatalogFile[FILENAME_MAX];

	if ( !pszCatalogFile )
		return false;
	
	if ( !logOpen( &hLog, "SQI", NULL, 20 ) )
		return false;
	logOn( hLog, true );

    /* GET DEFAULT PATH (home dir) */	
    pHomeDir	= "/home";		   		 				
    pPasswd		= (struct passwd *)getpwuid( nUserID );	
    if ( pPasswd != NULL )
        if ( (char *)pPasswd->pw_dir != NULL )
            pHomeDir = pPasswd->pw_dir;
    
    /* INIT HANDLE */    
    hSQI = malloc( sizeof(SQI) );

	hSQI->hLog				= hLog;
	hSQI->hStmtList			= lstOpen();
	hSQI->pszName			= 0;
	hSQI->pszDescription	= 0;
	hSQI->pszDefaultPath	= 0;

	lstSetFreeFunc( hSQI->hStmtList, _sqiCloseStmt );

    if ( *pszCatalogFile != '/' )
    {
        sprintf( szCatalogFile, "%s/%s", pHomeDir, pszCatalogFile );
        hSQI->pszCatalogFile	= (char*)strdup( szCatalogFile );
    }
    else
        hSQI->pszCatalogFile	= (char*)strdup( pszCatalogFile );

	/* sqi is single user so load table cache here.... */
	if ( !_sqiOpenDatabase( hSQI, pHomeDir, pszCatalogFile ) )
	{
		logPushMsg( hSQI->hLog, __FILE__, __FILE__, __LINE__, LOG_INFO, SQI_ERROR, "Had problems loading the database." );
        /* FREE MEM???? */
        return false;
	}
	
	/* finalize */
	*phSQI = hSQI;

	sprintf( hSQI->szMsg, "hSQI (%p) created for %s", hSQI, pszCatalogFile );
	logPushMsg( hSQI->hLog, __FILE__, __FILE__, __LINE__, LOG_INFO, SQI_SUCCESS, hSQI->szMsg );

	return true;
}



